package com.csc.library.dwr;

import java.sql.ResultSet;
import java.sql.Statement;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;

import com.csc.library.database.CscConnection;
import com.csc.library.database.StaticConfig;
import com.csc.library.database.StaticProperties;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.CscTime;
import com.csc.library.utilities.MyLog;
import com.csc.library.utilities.UProfile;

public class ManageOTIDS_dwr extends DWR_base  {
	String employeeid ;
	CscCalendar selectdate ;
	ArrayList<HashMap> arr = new ArrayList();
	StringBuilder emplist = new StringBuilder();
	UProfile up ;
	HashMap hmExtra = null;
	
	public ManageOTIDS_dwr() { }
	
	public void setUProfile(UProfile up){
		this.up = up;
	}
	
	public UProfile getUProfile(){
		if(this.up != null){
			return this.up;
		}else{
			this.setUProfile();
			this.up = super.up;
			return this.up;
		}
	}
	
	public void setEmployee(String id){
		this.employeeid = id;
	}
	
	public void clearData(){
		this.employeeid = "";
		this.selectdate = new CscCalendar();
		this.arr = new ArrayList();
		this.emplist = new StringBuilder();
	}
	
	public void setStartDate(String date){
		this.selectdate = new CscCalendar(date);
	}
	
	public ManageOTIDS_dwr(String empid, String stdate){
		this.employeeid = empid;
		this.selectdate = new CscCalendar(stdate);
	}
	
	public String getOTDayWeekStart(){
		return (String) StaticConfig.getConfigCompany(this.getUProfile(), "TAOT_ST");
	}
	
	private CscCalendar getStartOfWeek(CscCalendar date, int wkst){
		try {
			CscCalendar tmp = new CscCalendar(date.getYYYYMMDD());
			while(wkst != tmp.getDayOfWeek()){
				tmp.decDate();
			}
			return tmp ;
		}catch(Exception e){
			MyLog.error(this,e);
		}
		
		return new CscCalendar();
	}
	
	public CscCalendar getPeriodStart(){
		try {
			CscCalendar st = new CscCalendar(selectdate.getYYYYMMDD());
			return getStartOfWeek(st, Integer.parseInt(getOTDayWeekStart()));
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return new CscCalendar() ;
	}
	
	public CscCalendar getPeriodEnd(){
		CscCalendar en = null;
		try {
			en = getPeriodStart();
			en.incDate(6);
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return en ;
	}
	
	private void addEmplist(HashMap data){
		if(this.emplist.length() > 0){ this.emplist.append("#"); }
		
		StringBuilder sb = new StringBuilder();
		sb.append((String) data.get("companyid")).append(",");
		sb.append((String) data.get("employeeid")).append(",");
		sb.append((String) data.get("start_date")).append(",");
		sb.append((String) data.get("start_time")).append(",");
		sb.append((String) data.get("end_date")).append(",");
		sb.append((String) data.get("end_time")).append(",");
		sb.append((String) data.get("total_time")).append(",");
		sb.append((String) data.get("wf_ref_doc")).append("");
		
		this.emplist.append(sb.toString());
	}
	
	public String getEmplist(){
		return this.emplist.toString();
	}
	
	public String getDataOnPeriod(String employeeid, String date){
		try {
			this.setEmployee(employeeid);
			this.setStartDate(date);
			this.initOTRequested();
			
			return this.getEmplist();
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return "";
	}
	
	public String getReqOTTotal(String employeeid, String date){
		DecimalFormat dc = new DecimalFormat("#0.00");
		try {
			this.clearData();
			this.addExtraOT();	/* add ot prop from workflow page */
			this.setEmployee(employeeid);
			this.setStartDate(date);
			this.initOTRequested();
			
			return dc.format(calOTHour(arr));
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return "";
	}
	
	private float calOTHour(ArrayList arl){
		
		CscTime total = new CscTime();	/* ot total */
		float othr = 0;		/* ot on each period */
		HashMap otzone = new HashMap();
		CscCalendar st ;
		CscCalendar en ;

		try {
			if(arl != null){
				for(int i=0; i < arl.size(); i++){
					otzone = checkOTZone(i, arl);
					if(otzone != null && ((String)otzone.get("status")).equals("0")){
						st = new CscCalendar((String)otzone.get("start_date"));
						st.setCSCTime(Float.parseFloat((String)otzone.get("start_time")));
						
						en = new CscCalendar((String)otzone.get("end_date"));
						en.setCSCTime(Float.parseFloat((String)otzone.get("end_time")));
						
						othr = en.subForHour(st);
					}
					total.add(othr);
				}
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}

		return total.getFloat();
	}
	
	private HashMap checkOTZone(int idx, ArrayList arl){
		HashMap cur = null;
		try {
			if(arl == null) return null;

			cur = (HashMap) arl.get(idx);

			CscCalendar st = new CscCalendar((String)cur.get("start_date"));
			st.setCSCTime(Float.parseFloat((String)cur.get("start_time")));
			
			CscCalendar en = new CscCalendar((String)cur.get("end_date"));
			en.setCSCTime(Float.parseFloat((String)cur.get("end_time")));

			for(int j=0; j < arl.size(); j++){
				if(j != idx){	/* will not check itself */
					
					HashMap tmpot = (HashMap) arl.get(j) ;
					
					CscCalendar tmp_st = new CscCalendar((String)tmpot.get("start_date"));
					tmp_st.setCSCTime(Float.parseFloat((String)tmpot.get("start_time")));
					
					CscCalendar tmp_en = new CscCalendar((String)tmpot.get("end_date"));
					tmp_en.setCSCTime(Float.parseFloat((String)tmpot.get("end_time")));
					
					if(((String)tmpot.get("status")).equals("0")){	/* 0 active, 1 inactive (overlap with others) */
						if(st.afterEqualsDateTime(tmp_st) && en.beforeEqualsDateTime(tmp_en)){
							cur.put("status","1") ;	/* set inactive itself , ot hour will be 0 */
						}else if(st.beforeEqualsDateTime(tmp_st) && en.afterEqualsDateTime(tmp_en)){
							tmpot.put("status","1");	/* set inactive to tmpot */
						}else if(st.beforeEqualsDateTime(tmp_st) && en.afterDateTime(tmp_st) && en.beforeEqualsDateTime(tmp_en)){	/* end_time overlap with next shift */
							cur.put("end_date", (String) tmpot.get("start_date")) ;
							cur.put("end_time", (String) tmpot.get("start_time")) ;
						}else if(st.afterDateTime(tmp_st) && st.beforeEqualsDateTime(tmp_en) && en.afterDateTime(tmp_en)){	/* start time over with previous shift */
							cur.put("start_date", (String) tmpot.get("end_date")) ;
							cur.put("start_time", (String) tmpot.get("end_time")) ;
						}
					}
				}
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
		
		return cur ;
	}
	
	
	public void initOTRequested(){
		CscConnection conn = null;
    	Statement stmt = null;
    	ResultSet rs = null;
    	boolean autoCommit = true;
    	
    	try {
    		StringBuilder sql = new StringBuilder();
    		sql.append("select companyid,employeeid,start_date,start_time,end_date,end_time,total_time,wf_ref_doc");
    		sql.append(" from emp_create_list");
    		sql.append(" where employeeid = '").append(this.employeeid).append("'");
    		sql.append(" and start_date between '").append(this.getPeriodStart().getYYYYMMDD()).append("'");
    		sql.append(" and '").append(this.getPeriodEnd().getYYYYMMDD()).append("'");
    		sql.append(" order by start_date");
    		
			conn = (CscConnection) StaticProperties.myConnection.getConnection(this.getUProfile().get("dbname"));
			autoCommit = conn.getAutoCommit();
			conn.setAutoCommit(false);
			conn.setTransactionIsolation(conn.TRANSACTION_READ_UNCOMMITTED);

			stmt = conn.createStatement();
			rs = stmt.executeQuery(sql.toString());
			
			StringBuilder sb ;
			while(rs.next()){
				sb = new StringBuilder();
				
				HashMap hm = new HashMap();
				hm.put("companyid", rs.getString("companyid"));
				hm.put("employeeid", rs.getString("employeeid"));
				hm.put("start_date", rs.getString("start_date"));
				hm.put("start_time", rs.getString("start_time"));
				hm.put("end_date", rs.getString("end_date"));
				hm.put("end_time", rs.getString("end_time"));
				hm.put("total_time", rs.getString("total_time"));
				hm.put("wf_ref_doc", rs.getString("wf_ref_doc"));
				hm.put("status", "0");
				
				arr.add(hm);
				addEmplist(hm);
			}
			
        } catch (Exception e) {
            e.printStackTrace();           
        } finally {
    		try{
    			if(stmt != null) stmt.close();
	    		if(conn!=null){
	    			conn.setAutoCommit(autoCommit);
	    			conn.close();
	    		}
    		}catch(Exception e){
    			e.printStackTrace();
    		}
        }	
	}
	
	public String getWeekPeriod(){
		StringBuilder sb = new StringBuilder();
		try {
			CscCalendar fr = getStartOfWeek(new CscCalendar(), Integer.parseInt(this.getOTDayWeekStart()));
			fr.decDate(7*4); /* back date for 4 weeks */
			
			CscCalendar en = new CscCalendar(fr.getYYYYMMDD());
			en.incDate(6);	/* end of this week */
			
			for(int i=0; i < 8; i++){
				if(sb.length() > 0) sb.append("#");
				
				sb.append(fr.getYYYYMMDD()).append(",");
				sb.append(en.getYYYYMMDD());
				
				fr.incDate(7);
				en.incDate(7);
			}
			return sb.toString();
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return "";
	}
	
	public String getHint(){
		StringBuilder sb = new StringBuilder();
		try {
			HashMap prop ;
			for(int i=0; i < arr.size(); i++){
				prop = (HashMap) arr.get(i);
				sb.append((String) prop.get("start_date")).append(" ");
				sb.append((String) prop.get("start_time")).append(" ");
				sb.append((String) prop.get("end_date")).append(" ");
				sb.append((String) prop.get("end_date")).append(" ");
				sb.append((String) prop.get("total_time")).append(" ");
				sb.append((String) prop.get("wf_ref_doc")).append("<br>");
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
		return sb.toString();
	}
	
	public void addExtraReqOT(String req){
		try {
			String extra[] = req.split(",");
			
			hmExtra = new HashMap();
			hmExtra.put("companyid", "");
			hmExtra.put("employeeid", "");
			hmExtra.put("start_date", new CscCalendar(extra[0]).getYYYYMMDD());
			hmExtra.put("start_time", extra[1]);
			hmExtra.put("end_date", new CscCalendar(extra[2]).getYYYYMMDD());
			hmExtra.put("end_time", extra[3]);
			hmExtra.put("total_time", extra[4]);
			hmExtra.put("wf_ref_doc", "");
			hmExtra.put("status", "0");
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
	
	public void addExtraOT(){
		try {
			if(hmExtra != null){
				arr.add(hmExtra);
			}
		}catch(Exception e){
			MyLog.error(this,e);
		}
	}
}
