package com.csc.library.process;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.rmi.RemoteException;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Vector;

import com.csc.library.process.MyProcess;
import com.csc.library.session.DbInquiry;
import com.csc.library.session.InitialInquiry;
import com.csc.library.utilities.CscCalendar;

public class Export2Sap extends MyProcess {
	private DecimalFormat id = new DecimalFormat("00000000"); 
	private DecimalFormat time = new DecimalFormat("00.0000"); 
	private CscCalendar cc = new CscCalendar();
	private StringBuilder sb = new StringBuilder();
	private File f;
	private FileOutputStream fo;
	private OutputStreamWriter os; 
	private BufferedWriter bf;
	private HashMap<String,String> cumulative = new HashMap<String, String>();

	public Vector<String> getProcessList() {
		Vector<String> vector = new Vector<String>();
		vector.add("all");
		return vector;
	}
	
	public void process() throws Exception {
		processOT();
		processLeave();
		processSwipeCard();
	}
	private DbInquiry getInquiry(String datatype){
        DbInquiry inq = new InitialInquiry(this.upf).getDbInquiry("data2sap");
        try {        	
			inq.setColumn("*");
			inq.setFilter("datatype='"+datatype+"'");
	        if("SWIPECARD".equalsIgnoreCase(datatype)){
	        	inq.setOrderBy("PERNR,BEGDA");
	        }
	        inq.refresh();
	        return inq;
		} catch (RemoteException e) {
			e.printStackTrace();
			return null;
		}
	}
	
	private void processOT(){		
		try{
			DbInquiry rs = getInquiry("OT");
			
			f = new File("D:\\DATA\\OT"+format(1,new CscCalendar().getDDMMYYYY())+".txt");
			
			fo = new FileOutputStream(f);
			os = new OutputStreamWriter(fo);
			bf = new BufferedWriter(os);
			
			bf.write("PERNR,BEGDA,LGART,ANZHL,ZEINH\r\n");
			while(rs.next()&&!this.isCancle()){
				if(!"".equals(rs.getString("LGART1"))){
					sb.setLength(0);			
					bf.write(sb.append(format(0,rs.getString("PERNR")))
							.append(",")
							.append(format(1,rs.getString("BEGDA")))
							.append(",2010,")
							.append(rs.getString("ANZHL1"))
							.append(",")
							.append(rs.getString("ZEINH"))
							.append("\r\n")
							.toString());
				}
				if(!"".equals(rs.getString("LGART15"))){
					sb.setLength(0);
					bf.write(sb.append(format(0,rs.getString("PERNR")))
							.append(",")
							.append(format(1,rs.getString("BEGDA")))
							.append(",2015,")
							.append(rs.getString("ANZHL15"))
							.append(",")
							.append(rs.getString("ZEINH"))
							.append("\r\n")
							.toString());
				}
				if(!"".equals(rs.getString("LGART3"))){
					sb.setLength(0);
					bf.write(sb.append(format(0,rs.getString("PERNR")))
							.append(",")
							.append(format(1,rs.getString("BEGDA")))
							.append(",2030,")
							.append(rs.getString("ANZHL3"))
							.append(",")
							.append(rs.getString("ZEINH"))
							.append("\r\n")
							.toString());
				}
			}
						
			bf.close();
			os.close();
			fo.close();			
			
			}catch(Exception ex){
				ex.printStackTrace();
		}
	}
	private void processLeave(){		
		try{
			DbInquiry rs = getInquiry("LEAVE");
			
			f = new File("D:\\DATA\\LEAVE"+format(1,new CscCalendar().getDDMMYYYY())+".txt");
			
			fo = new FileOutputStream(f);
			os = new OutputStreamWriter(fo);
			bf = new BufferedWriter(os);
			
			bf.write("PERNR,BEGDA,ENDDA,AWART,BEGUZ,ENDUZ\r\n");
			while(rs.next()&&!this.isCancle()){
					sb.setLength(0);			
					bf.write(sb.append(format(0,rs.getString("PERNR")))
							.append(",")
							.append(format(1,rs.getString("BEGDA")))
							.append(",")
							.append(format(1,rs.getString("ENDDA")))
							.append(",")
							.append(rs.getString("AWART"))
							.append(",")
							.append(format(2,rs.getString("BEGUZ")))
							.append(",")
							.append(format(2,rs.getString("ENDUZ")))
							.append("\r\n")
							.toString());
			}
						
			bf.close();
			os.close();
			fo.close();			
			
			}catch(Exception ex){
				ex.printStackTrace();
		}
	}
	private void processSwipeCard(){		
		try{
			DbInquiry rs = getInquiry("SWIPECARD");
			
			f = new File("D:\\DATA\\SWIPECARD"+format(1,new CscCalendar().getDDMMYYYY())+".txt");
			
			fo = new FileOutputStream(f);
			os = new OutputStreamWriter(fo);
			bf = new BufferedWriter(os);
			
			
			if(rs.next()){
				bf.write("PERNR,BEGDA,ENDDA,AWART,BEGUZ,ENDUZ\r\n");
				putNewCumulative(rs);
			}
			while(rs.next()&&!this.isCancle()){
				cc.setCalendar(rs.getString("BEGDA"));
				if(rs.getString("PERNR").equals(cumulative.get("PERNR"))){
					if(rs.getString("BEGDA").equals(cumulative.get("BEGDA"))){
						cumulative.put("ENDUZ", rs.getString("ENDUZ"));						
					}
					else if(cc.countDay(cumulative.get("LASTDATE"))-1==1){
						if(rs.getString("BEGUZ").equals(cumulative.get("BEGUZ"))){
							continue;
						}
						else if(rs.getString("ENDUZ").equals(cumulative.get("ENDUZ"))){
							cumulative.put("LASTDATE", rs.getString("BEGDA"));
							cumulative.put("ENDDA", rs.getString("ENDDA"));
						}
						else{
							writeCumulative();	
							putNewCumulative(rs);
						}
					}		
					else{
						writeCumulative();
						putNewCumulative(rs);
					}				
				}else{
					writeCumulative();
					putNewCumulative(rs);
				}
				
			}
			writeCumulative();
				
						
			bf.close();
			os.close();
			fo.close();			
			
			}catch(Exception ex){
				ex.printStackTrace();
		}
	}
	private void writeCumulative(){
		sb.setLength(0);		
		try {
			bf.write(sb.append(format(0,cumulative.get("PERNR")))
							.append(",")
							.append(format(1,cumulative.get("BEGDA")))
							.append(",")
							.append(format(1,cumulative.get("ENDDA")))
							.append(",")
							.append(cumulative.get("AWART"))
							.append(",")
							.append(format(2,cumulative.get("BEGUZ")))
							.append(",")
							.append(format(2,cumulative.get("ENDUZ")))
							.append("\r\n")
							.toString());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	private void putNewCumulative(DbInquiry rs){
		try {
			cumulative.put("PERNR", rs.getString("PERNR"));
			cumulative.put("BEGDA", rs.getString("BEGDA"));
			cumulative.put("ENDDA", rs.getString("ENDDA"));
			cumulative.put("AWART", rs.getString("AWART"));
			cumulative.put("BEGUZ", rs.getString("BEGUZ"));
			cumulative.put("ENDUZ", rs.getString("ENDUZ"));
			cumulative.put("LASTDATE", rs.getString("BEGDA"));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private String format(int type,String data){
		if(type==0){//employeeid
			try{
				int newData = Integer.parseInt(data);
				return 	id.format(newData).toString();
			}catch(NumberFormatException e){
				System.out.println("Wrong Employee id format "+data);
				e.printStackTrace();
			}
		}
		else if(type==1){//date
			cc.setCalendar(data);
			return 	cc.getDDMMYYYY().replace("-", "");
		}
		else if(type==2){//time
			try{
				double newData = Double.parseDouble(data);
				return 	time.format(newData).toString().replace(".", "");
			}catch(NumberFormatException e){
				System.out.println("Wrong time format "+data);
				e.printStackTrace();
			}
		}
		return data;
	}
}
